==> chift-0.3.9.tar.gz: chift-0.3.9/chift/README.md <==

# Development

## A model is changed in chift

As models are kept in sync thanks to openapi, you need to regenerate `openapi.py` and make sure there is no link error in `models.py`.

We generate the models used in this SDK thanks to  [datamodel-code-generator](https://github.com/koxudaxi/datamodel-code-generator/)

```bash
PYTHONPATH=. datamodel-codegen --input ./openapi.json --output openapi.py --output-model-type pydantic_v2.BaseModel --custom-formatters chift_formatter
```

Then, those models are mapped to the one used in the python SDK in ```models.py```


## A route is added in chift

If there is a new route, here are the needed changes:

- Generate the new model in `openapi.py` thanks to an update in `openapi.json` and `datamodel-codegen`
- Create the link to openapi models and models used in this SDK in `models.py`
- Add the model to the relevant folder in `/models`
- Add the relevant mixins (create, update, list, read,...)
- Add tests for the model in `/tests`


## A route is changed in chift

- Find the exising model under `/models` and update its definition

## Testing

When writing or updating tests, we mock all Chift client calls by replacing responses with fixtures. Adding a new model typically requires only to:

 - Write tests under the /tests directory (e.g., tests/test_<resource>.py), using pytest.

 - Add fixtures in tests/fixtures/<resource>.py, containing example JSON responses. Fixtures can be:

   - Real responses captured from the Chift backend.

   - Manually generated sample objects following the OpenAPI schema.

Decorate the test with @pytest.mark.mock_chift_response(...), passing the fixture constants in the order of client calls:
```python
from tests.fixtures import <resource> as fixtures

@pytest.mark.mock_chift_response(
    fixtures.<RESOURCE_CREATE>,
    fixtures.<RESOURCE_GET>
)
def test_<resource>(<resource>_consumer):
    # Create
    created = <resource>_consumer.<domain>.<Resource>.create(data)
    assert created.id

    # Retrieve
    fetched = <resource>_consumer.<domain>.<Resource>.get(str(created.id))
    assert fetched == created
```
- Run and verify that the test passes and no real HTTP calls are made.

==> chift-0.4.2.tar.gz: chift-0.4.2/chift/README.md <==

# Development

## A model is changed in chift

As models are kept in sync thanks to openapi, you need to regenerate `openapi.py` and make sure there is no link error in `models.py`.

We generate the models used in this SDK thanks to  [datamodel-code-generator](https://github.com/koxudaxi/datamodel-code-generator/)

```bash
PYTHONPATH=. datamodel-codegen --input ./openapi.json --output openapi.py --output-model-type pydantic_v2.BaseModel --custom-formatters chift_formatter --use-double-quotes
```

Then, those models are mapped to the one used in the python SDK in ```models.py```


## A route is added in chift

If there is a new route, here are the needed changes:

- Generate the new model in `openapi.py` thanks to an update in `openapi.json` and `datamodel-codegen`
- Create the link to openapi models and models used in this SDK in `models.py`
- Add the model to the relevant folder in `/models`
- Add the relevant mixins (create, update, list, read,...)
- Add tests for the model in `/tests`


## A route is changed in chift

- Find the exising model under `/models` and update its definition

## Testing

When writing or updating tests, we mock all Chift client calls by replacing responses with fixtures. Adding a new model typically requires only to:

 - Write tests under the /tests directory (e.g., tests/test_<resource>.py), using pytest.

 - Add fixtures in tests/fixtures/<resource>.py, containing example JSON responses. Fixtures can be:

   - Real responses captured from the Chift backend.

   - Manually generated sample objects following the OpenAPI schema.

Decorate the test with @pytest.mark.mock_chift_response(...), passing the fixture constants in the order of client calls:
```python
from tests.fixtures import <resource> as fixtures

@pytest.mark.mock_chift_response(
    fixtures.<RESOURCE_CREATE>,
    fixtures.<RESOURCE_GET>
)
def test_<resource>(<resource>_consumer):
    # Create
    created = <resource>_consumer.<domain>.<Resource>.create(data)
    assert created.id

    # Retrieve
    fetched = <resource>_consumer.<domain>.<Resource>.get(str(created.id))
    assert fetched == created
```
- Run and verify that the test passes and no real HTTP calls are made.
